#include	<unistd.h>
#include	<stdio.h>
#include	<sys/fcntl.h>
#include	<stdlib.h>

#include	<yz/window.h>
#include	<yz/draw.h>
#include	<yzuitk/scrollbar.h>
#include	"embedded-view.h"
#include	"font-info.h"
#include	"lkb-tree.h"
#include	"lkb-avm.h"
#include	"lkb-chart.h"
#include	"lkb-protocol.h"

void	init_view(embedded_view_t	*view, int	x, int	y, int	dx, int	dy)
{
	view->w_x1 = x;
	view->w_y1 = y;

	view->use_width = dx - x;
	view->use_height = dy - y;
	view->vis_width = view->use_width - 15;
	view->vis_height = view->use_height - 15;

	view->m_x = view->m_y = -1;

	//printf("initview (%d %d %d %d) %d x %d\n", x, y, dx, dy, view->vis_width, view->vis_height);
	view->hscroll = yzAddScrollBar(x,y+view->vis_height, view->vis_width, 15, 1);
	view->vscroll = yzAddScrollBar(x+view->vis_width,y, 15, view->vis_height, 2);
	yzSetScrollBarPageSize(view->hscroll, view->vis_width);
	yzSetScrollBarPageSize(view->vscroll, view->vis_height);
	yzSetScrollBarMax(view->hscroll, view->width);
	yzSetScrollBarMax(view->vscroll, view->height);
	yzSetScrollBarPosition(view->hscroll, 0);
	yzSetScrollBarPosition(view->vscroll, 0);
	view->scroll_area = yzAddScrollArea(x, y,
							view->vis_width, view->vis_height,
							view->update, view->event,
							(void*)view);
	yzSetScrollBarArea(view->hscroll, view->scroll_area);
	yzSetScrollBarArea(view->vscroll, view->scroll_area);
	yzPenColor(30000, 30000, 30000);
	yzRect(x+view->vis_width, y+view->vis_height, x+view->use_width-1, y+view->use_height-1);
}

int	embedded_avm_view_drag(char	*type, int	x, int	y)
{
	embedded_view_t	*view;

	view = yzGetWindowData("embedded_view");
	if(!view)
	{
		fprintf(stderr, "embedded_avm_view_drag callback: no view!\n");
		return 0;
	}

	if(x<view->w_x1 || y<view->w_y1 ||
		x>=view->w_x1+view->vis_width ||
		y>=view->w_y1+view->vis_height)
		x=y=-1;
	else
	{
		x += yzGetScrollBarPosition(view->hscroll) - view->w_x1;
		y += yzGetScrollBarPosition(view->vscroll) - view->w_y1;
	}

	return avm_view_drag(view, type, x, y);
}

void	embedded_view_event(event_t	ev)
{
	embedded_view_t	*view;
	window_t	*win;
	int				x, y;

	view = yzGetWindowData("embedded_view");
	if(!view)
	{
		fprintf(stderr, "embedded_view_event callback: no view!\n");
		return;
	}

	//printf("got an embedded view event for window %s\n", yzGetSelectedWindow()->name);
	switch(ev.type)
	{
		case	YZ_KEY_DOWN:
			switch(ev.key)
			{
				case	'q':
					_close:
					win = yzGetSelectedWindow();
					close_view(view);
					deregister_window_manager(win);
					yzDeleteWindow(win);
					return;
				case	'p':
					if(view->postscript)view->postscript(view);
					break;
				case	'l':
					if(view->latex)view->latex(view);
					break;
				default:
					if(view->event)view->event(view, ev);
					break;
			}
			break;
		case	YZ_WINDOW_RESIZE:
			//printf("resized window to %dx%d\n", ev.x, ev.y);
			resize_view(view, ev.x, ev.y);
			yzUpdateBuffer(YZ_BUFFER_FRONT);
			break;
		case	YZ_WINDOW_CLOSE:
			goto _close;
	}
	if(view->mouse_moved)
	{
		if(-1 == yzQueryPointer(&x, &y))x=y=-1;
		else
		{
			if(x<view->w_x1 || y<view->w_y1 ||
				x>=view->w_x1+view->vis_width ||
				y>=view->w_y1+view->vis_height)
				x=y=-1;
			else
			{
				x += yzGetScrollBarPosition(view->hscroll) - view->w_x1;
				y += yzGetScrollBarPosition(view->vscroll) - view->w_y1;
			}
		}
		if(x != view->m_x || y != view->m_y)
		{
			view->m_x = x;
			view->m_y = y;
			view->mouse_moved(view, x, y);
		}
	}
}

void	resize_view(embedded_view_t	*view, int	width, int	height)
{
	int	x=view->w_x1, y=view->w_y1;
	// XXX should also save bot/right margins

	view->use_width = width - x;	view->use_height = height - y;
	view->vis_width = view->use_width - 15;
	view->vis_height = view->use_height - 15;
	//printf("resizeview (%d %d %d %d) %d x %d\n", x, y, width, height, view->vis_width, view->vis_height);
	//printf(" -- "); show_clip();
	yzResizeScrollArea(view->scroll_area, x, y, view->vis_width, view->vis_height);
	yzResizeScrollBar(view->hscroll, x,y+view->vis_height, view->vis_width, 15);
	yzResizeScrollBar(view->vscroll, x+view->vis_width,y, 15, view->vis_height);
	yzSetScrollBarPageSize(view->hscroll, view->vis_width);
	yzSetScrollBarPageSize(view->vscroll, view->vis_height);
	//printf("before drawscrollarea: "); show_clip();
	yzDrawScrollArea(view->scroll_area);
	yzPenColor(30000, 30000, 30000);
	yzRect(x+view->vis_width, y+view->vis_height, x+view->use_width-1, y+view->use_height-1);
}

void	close_view(embedded_view_t	*view)
{
	yzRemoveScrollBar(view->hscroll);
	yzRemoveScrollBar(view->vscroll);
	yzRemoveScrollArea(view->scroll_area);
	if(view->close)view->close(view);
	free(view);
}

void	scroll_embedded_view_to(embedded_view_t	*view, int	x, int	y)
{
	yzPopClipRect();
	yzPopOrigin();

	yzBufferMode(YZ_BUFFER_BOTH);
	yzSetScrollBarPosition(view->hscroll, x - view->vis_width/2);
	yzSetScrollBarPosition(view->vscroll, y - view->vis_height/2);

	yzRecomputeScrollAreaClipping(view->scroll_area);
}

void	set_embedded_view_size(embedded_view_t	*view, int	w, int	h)
{
	int	cx1, cy1, cx2, cy2, ox, oy;

	yzPopOrigin2p(&ox, &oy);
	yzPopClipRect4p(&cx1, &cy1, &cx2, &cy2);
	yzBufferMode(YZ_BUFFER_BOTH);
	if(view->width != w)
	{
		view->width = w;
		yzSetScrollBarMax(view->hscroll, w);
	}
	if(view->height != h)
	{
		view->height = h;
		yzSetScrollBarMax(view->vscroll, h);
	}
	yzPushClipRect(cx1, cy1, cx2, cy2);
	yzPushOrigin(ox, oy);
}

extern font_info		avm_bar_font, tree_bar_font, mtree_bar_font, chart_bar_font;

void	browse_tree(int	id, lkb_tree	*tree, char	*title)
{
	extern embedded_view_t	prototype_tree_view;
	window_t				*window;
	embedded_view_t			*tree_view;
	int		w_width, w_height;

	tree_view = calloc(sizeof(*tree_view), 1);
	*tree_view = prototype_tree_view;
	tree_view->private = 0;
	if(tree_view->init(tree_view, id, tree))return;

	w_width = tree_view->width + 16;
	w_height = tree_view->height + 16 + tree_bar_font.size*1.3+1;
	if(w_width>800)w_width = 800;
	if(w_height>600)w_height = 600;
	window = yzNewWindow(title, w_width, w_height);
	yzSelectWindow(window);
	init_view(tree_view, 0, tree_bar_font.size*1.3+1, window->width, window->height);
	yzPutWindowData("embedded_view", tree_view);
	register_window_manager(window, embedded_view_event, 0, wmTypeTree);

	free(title);
}

void	browse_mtree(int	count, int	*ids, lkb_tree	**trees, char	**titles, char	*title)
{
	extern embedded_view_t	prototype_mtree_view;
	window_t				*window;
	embedded_view_t			*tree_view;
	int		w_width, w_height;

	tree_view = calloc(sizeof(*tree_view), 1);
	*tree_view = prototype_mtree_view;
	tree_view->private = 0;
	if(tree_view->init(tree_view, count, ids, trees, titles))return;

	w_width = tree_view->width + 16;
	w_height = tree_view->height + 16 + mtree_bar_font.size*1.3+1;
	if(w_width>800)w_width = 800;
	if(w_height>600)w_height = 600;
	window = yzNewWindow(title, w_width, w_height);
	yzSelectWindow(window);
	init_view(tree_view, 0, mtree_bar_font.size*1.3+1, window->width, window->height);
	yzPutWindowData("embedded_view", tree_view);
	register_window_manager(window, embedded_view_event, 0, wmTypeTree);

	free(title);
}

void	browse_avm(int	id, lkb_avm	*avm, char	*title, lui_list	*failures)
{
	extern embedded_view_t	prototype_avm_view;
	window_t				*window;
	embedded_view_t			*avm_view;
	int		w_width, w_height;

	avm_view = calloc(sizeof(*avm_view), 1);
	*avm_view = prototype_avm_view;
	avm_view->private = 0;
	if(avm_view->init(avm_view, id, avm, failures))return;

	w_width = avm_view->width + 16;
	w_height = avm_view->height + 16 + avm_bar_font.size*1.3;
	if(w_width>900)w_width = 900;
	if(w_height>700)w_height = 700;
	window = yzNewWindow(title, w_width, w_height);
	yzSelectWindow(window);
	init_view(avm_view, 0, avm_bar_font.size*1.3, window->width, window->height);
	yzPutWindowData("embedded_view", avm_view);
	register_window_manager(window, embedded_view_event, embedded_avm_view_drag, wmTypeAVM);

	yzPushOrigin(0,0);
	yzPushClipRect(0,0,window->width,window->height);
	prescroll_avm_view(avm_view);
	yzPopOrigin();
	yzPopClipRect();

	free(title);
}

void	browse_chart(int	id, lkb_chart	*chart, char	*title)
{
	extern embedded_view_t	prototype_chart_view;
	window_t				*window;
	embedded_view_t			*chart_view;
	int		w_width, w_height;

	chart_view = calloc(sizeof(*chart_view), 1);
	*chart_view = prototype_chart_view;
	chart_view->private = 0;
	if(chart_view->init(chart_view, id, chart, title))return;

	w_width = chart_view->width + 16;
	//w_height = chart_view->height + 16 + 20;
	w_height = chart_view->height + 16 + chart_bar_font.size*1.3;
	if(w_width>900)w_width = 900;
	if(w_height>700)w_height = 700;
	window = yzNewWindow(title, w_width, w_height);
	yzSelectWindow(window);
	init_view(chart_view, 0, chart_bar_font.size*1.3, window->width, window->height);
	yzPutWindowData("embedded_view", chart_view);
	register_window_manager(window, embedded_view_event, 0, wmTypeChart);

	// chart owns 'title', don't free() it.
}

void	browse_xtext(int	id, lui_xtext	*text, int	nmenus, struct lui_xtext_menu	**menus, char	*title)
{
	extern embedded_view_t	prototype_xtext_view;
	window_t				*window;
	embedded_view_t			*xtext_view;
	int						w_width, w_height;

	xtext_view = calloc(sizeof(*xtext_view), 1);
	*xtext_view = prototype_xtext_view;
	xtext_view->private = 0;
	if(xtext_view->init(id, xtext_view, text, nmenus, menus))return;

	w_width = xtext_view->width + 16;
	//w_height = xtext_view->height + 16 + 20;
	w_height = xtext_view->height + 16;
	if(w_width>900)w_width = 900;
	if(w_height>700)w_height = 700;
	window = yzNewWindow(title, w_width, w_height);
	yzSelectWindow(window);
	init_view(xtext_view, 0, 0, window->width, window->height);
	yzPutWindowData("embedded_view", xtext_view);
	register_window_manager(window, embedded_view_event, 0, wmTypeXText);

	free(title);
}

void	do_chart_event(int	cid, char	*what, int	edge)
{
	int	i;
	struct embedded_view	*em;

	for(i=0;i<awm;i++)if(wms[i].w && wms[i].type==wmTypeChart)
	{
		yzSelectWindow(wms[i].w);
		em = yzGetWindowData("embedded_view");
		if(em)chart_special_event(em, cid, what, edge);
	}
}
